
from jinnang.io.system import suppress_c_stdout_stderr, suppress_stdout_stderr, get_worker_num_for_io_bounded_task, create_relative_symlink, safe_delete, safe_move, copy_with_meta, inplace_overwrite_meta
from jinnang.io.file import is_bad_llm_caption, is_bad_folder_name